import os
import re
import sys
import logging
import requests

from typing import Any, Union
from collections.abc import Iterable, Iterator
from datetime import datetime, timedelta


DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"


# Setup root logger
LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO").upper()
logging.basicConfig(
    stream=sys.stdout,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    level=LOG_LEVEL,
)
LOG = logging.getLogger(__name__)


def camel_to_snake(s: str, upper: bool = True) -> str:
    """
    Convert from camel to snake case.

    Args:
        s (str): A string
        upper (bool): Convert to upper case. Defaults to true

    Returns:
        The snake case version of the input string
    """
    s = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', s)
    if upper:
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).upper()
    else:
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).lower()


def batch_list(iterable: Iterable[Any], batch_size: int = 1) -> Iterator[Union[Iterable[Any], Any]]:
    """
    Batch an iterable.

    Args:
        iterable (Iterable): An iterable to be batched
        batch_size (int): The batch size. Defauts to 1

    Yields:
        Iterator[Iterable[Any] | Any]: A subset of the iterable
    """
    length = len(iterable)
    for ndx in range(0, length, batch_size):
        yield iterable[ndx:min(ndx + batch_size, length)]


def get_utc_timestamp(s: str, **kwargs) -> int:
    """
    Get utc timestamp

    Args:
        s (str): A string representation of a date
        delta (int): Number of days to be added to the date. Defaults to 0

    Returns:
        The timestamp of the provided date adjusted for the delta value
    """
    return int(datetime.timestamp(datetime.strptime(s, DATE_FORMAT) + timedelta(**kwargs)) * 1000) if isinstance(s, str) else s


def paginate(url: str, headers: dict, data: list, skip: int = 0, take: int = 100):
    # TODO: Change pageIndex/pageSize to be configurable
    new = requests.get(f"{url}?pageIndex={skip}&pageSize={take}", headers=headers).json()
    if len(new) == 0:
        return data
    else:
        data.extend(new)
        return paginate(url, headers, data, skip + 1)


def token_request(
    tenant_id: str,
    client_id: str,
    client_secret: str,
    grant_type: str,
    scopes: str
) -> dict[str, Any]:
    """
    Get OAuth2 token from AAD.
    Returns:
        JSON response from AAD.
    """
    return requests.post(
        f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/token",
        data={
            "tenant": tenant_id,
            "client_id": client_id,
            "client_secret": client_secret,
            "grant_type": grant_type,
            "scope": scopes,
        }
    ).json()



import os
import sys
import logging
import pandas as pd

from markus_pettersen.general import camel_to_snake


# Setup root logger
LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO").upper()
logging.basicConfig(
    stream=sys.stdout,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    level=LOG_LEVEL,
)
LOG = logging.getLogger(__name__)


def unpack_nested_column(df: pd.DataFrame, column: str) -> pd.DataFrame:
    """
    Unpack nested column from a dataframe.
    Removes the nested column.

    Args:
        df (pd.DataFrame): Dataframe with of a CDF object
        column (List[str]): Column to unpack

    Returns:
        pd.Dataframe: Dataframe with the column unpacked
    """
    return pd.concat([df, pd.json_normalize(df[column])], axis=1).drop(columns=column)


def get_batch(df: pd.DataFrame, ids: dict[str, list[str]], start_key: str) -> pd.DataFrame:
    """
    Get batch given the provided ids

    Args:
        df (pd.DataFrame): The full dataframe
        ids (dict[str, list[str]]): Dictionary with ids and the name of the corresponding column
        start_key (str): The name of the first id column

    Returns:
        pd.DataFrame: A batch of the dataframe
    """
    comparison = df[start_key].isin(ids[start_key])
    for k, v in ids.items():
        if k == start_key:
            continue
        comparison &= df[k].isin(v)
    return df.loc[comparison]


def fix_columns(df: pd.DataFrame, target_columns: list[str]) -> pd.DataFrame:
    """
    Fix columns with a .value suffix

    Args:
        df (pd.DataFrame): A dataframe
        target_columns (list[str]): The columns to keep

    Returns:
        pd.DataFrame: Dataframe with the fixed columns
    """
    df.columns = [
        camel_to_snake(col.replace('.value', ''))
        for col in df.columns
    ]
    return df[[col for col in target_columns if col in df.columns]]


def filter_reference(df: pd.DataFrame, col: str, values: list):
    return df.loc[df[col].isin(values)]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['markus_pettersen']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5.3,<2.0.0']

setup_kwargs = {
    'name': 'markus-pettersen',
    'version': '0.1.1',
    'description': 'Random useful python stuff I sometimes use',
    'long_description': "# utils\nI'm tired of making these functions all the time\n",
    'author': 'Markus Pettersen',
    'author_email': 'mp.markus94@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/MPettersen/markus',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
